@@ -103,17 +103,11 @@ def worker_def(A):
 
 
 class Point_Output_Maker_Class(object):
-    def __init__(self, image_size_rows, image_size_cols, slice_info, PixelSize, contour_dict, mv, shift_list,
-                 RS, ds):
+    def __init__(self, image_size_rows, image_size_cols, PixelSize, contour_dict, RS):
         self.image_size_rows, self.image_size_cols = image_size_rows, image_size_cols
-        self.slice_info = slice_info
         self.PixelSize = PixelSize
-        self.mv = mv
-        self.shift_list = shift_list
         self.contour_dict = contour_dict
         self.RS = RS
-        self.ds = ds
-        self.shift_list = shift_list
 
     def make_output(self, annotation, i):
         self.contour_dict[i] = []
@@ -133,8 +127,8 @@ class Point_Output_Maker_Class(object):
 
 
 class DicomReaderWriter:
-    def __init__(self, rewrite_RT_file=False, delete_previous_rois=True,Contour_Names=None,
-                 template_dir=None, get_images_mask=True, arg_max=True,
+    def __init__(self, rewrite_RT_file=False, delete_previous_rois=True, Contour_Names=None,
+                 template_dir=None, get_images_mask=True, arg_max=True, create_new_RT=True,
                  associations={},desc='',iteration=0, get_dose_output=False, flip_axes=(False, False, False), **kwargs):
         '''
         :param rewrite_RT_file: Boolean, should we re-write the RT structure
@@ -152,6 +146,7 @@ class DicomReaderWriter:
         '''
         self.get_dose_output = get_dose_output
         self.flip_axes = flip_axes
+        self.create_new_RT = create_new_RT
         self.associations = associations
         self.set_contour_names(Contour_Names)
         self.set_associations(associations)
@@ -174,6 +169,7 @@ class DicomReaderWriter:
 
     def __reset__(self):
         self.all_RTs = {}
+        self.RTs_with_ROI_Names = {}
         self.all_rois = []
         self.all_paths = []
         self.paths_with_contours = []
@@ -206,24 +202,27 @@ class DicomReaderWriter:
     def set_iteration(self, iteration=0):
         self.iteration = str(iteration)
 
-    def down_folder(self, input_path, reset=True):
-        files = []
-        dirs = []
-        file = []
+    def down_folder(self, input_path):
         for root, dirs, files in os.walk(input_path):
-            break
-        for val in files:
-            if val.find('.dcm') != -1:
-                file = val
-                break
-        if file and input_path:
-            self.all_paths.append(input_path)
-            self.Make_Contour_From_directory(input_path)
-        for dir in dirs:
-            new_directory = os.path.join(input_path, dir)
-            self.down_folder(new_directory)
+            dicom_files = [i for i in files if i.endswith('.dcm')]
+            if dicom_files:
+                self.all_paths.append(root)
+                self.Make_Contour_From_directory(root)
         return None
 
+    def where_are_RTs(self, ROIName):
+        if ROIName.lower() in self.RTs_with_ROI_Names:
+            print('Contours of {} are located:'.format(ROIName.lower()))
+            for path in self.RTs_with_ROI_Names[ROIName.lower()]:
+                print(path)
+        else:
+            print('{} was not found within the set, check spelling or list all rois'.format(ROIName))
+
+    def list_rois(self):
+        print('The following ROIs were found')
+        for roi in self.all_rois:
+            print(roi)
+
     def make_array(self, PathDicom):
         self.PathDicom = PathDicom
         self.lstFilesDCM = []
@@ -257,6 +256,8 @@ class DicomReaderWriter:
         else:
             self.dicom_names = self.reader.GetGDCMSeriesFileNames(self.PathDicom)
             self.reader.SetFileNames(self.dicom_names)
+            self.RefDs = pydicom.read_file(self.dicom_names[0])
+            self.ds = pydicom.read_file(self.dicom_names[0])
             self.get_images()
             image_files = [i.split(PathDicom)[1][1:] for i in self.dicom_names]
             RT_Files = [os.path.join(PathDicom, file) for file in fileList if file not in image_files]
@@ -272,8 +273,7 @@ class DicomReaderWriter:
                     self.RDs_in_case[lstRSFile] = []
                 elif modality.lower().find('struct') != -1:
                     self.RTs_in_case[lstRSFile] = []
-            self.RefDs = pydicom.read_file(self.dicom_names[0])
-            self.ds = pydicom.read_file(self.dicom_names[0])
+
         self.all_contours_exist = False
         self.rois_in_case = []
         self.all_RTs.update(self.RTs_in_case)
@@ -347,6 +347,10 @@ class DicomReaderWriter:
             if Structures.ROIName not in self.rois_in_case:
                 self.rois_in_case.append(Structures.ROIName)
                 rois_in_structure[Structures.ROIName] = Structures.ROINumber
+            if Structures.ROIName.lower() not in self.RTs_with_ROI_Names:
+                self.RTs_with_ROI_Names[Structures.ROIName.lower()] = [self.lstRSFile]
+            else:
+                self.RTs_with_ROI_Names[Structures.ROIName.lower()].append(self.lstRSFile)
         self.all_RTs[self.lstRSFile] = rois_in_structure
         self.RTs_in_case[self.lstRSFile] = rois_in_structure
 
@@ -422,17 +426,9 @@ class DicomReaderWriter:
 
     def get_images(self):
         self.dicom_handle = self.reader.Execute()
-        self.mv = [float(i) for i in self.reader.GetMetaData(0, "0020|0037").split('\\')]
-        self.shift_list = [[float(i) for i in self.reader.GetMetaData(j, "0020|0032").split('\\')]
-                           for j in range(len(self.reader.GetFileNames()))] #ShiftRows, ShiftCols, ShiftZBase
-        # self.shift_list = [self.dicom_handle[:, :, i:i+1].GetOrigin() for i in range(self.dicom_handle.GetSize()[-1])]
         sop_instance_UID_key = "0008|0018"
         self.SOPInstanceUIDs = [self.reader.GetMetaData(i, sop_instance_UID_key) for i in
                                 range(self.dicom_handle.GetDepth())]
-        slice_location_key = "0020|0032"
-        self.slice_info = [self.reader.GetMetaData(i, slice_location_key).split('\\')[-1] for i in
-                           range(self.dicom_handle.GetDepth())]
-        self.patient_position = self.reader.GetMetaData(0, "0018|5100")
         if max(self.flip_axes):
             flipimagefilter = sitk.FlipImageFilter()
             flipimagefilter.SetFlipAxes(self.flip_axes)
@@ -477,9 +473,20 @@ class DicomReaderWriter:
         mask[fill_row_coords, fill_col_coords] = True
         return mask
 
-    def with_annotations(self, annotations, output_dir, ROI_Names=None):
+    def prediction_array_to_RT(self, prediction_array, output_dir, ROI_Names=None):
+        '''
+        :param prediction_array: numpy array of prediction, expected shape is [#Images, Rows, Cols, #Classes + 1]
+        :param output_dir: directory to pass RT structure to
+        :param ROI_Names: list of ROI names equal to the number of classes
+        :return:
+        '''
+        if self.create_new_RT:
+            self.use_template()
         assert ROI_Names is not None, 'You need to provide ROI_Names'
-        annotations = np.squeeze(annotations)
+        assert prediction_array.shape[-1] == len(ROI_Names) + 1, 'Your last dimension of prediction array should be' \
+                                                                 ' equal  to the number or ROI_names minus 1, channel' \
+                                                                 ' 0 is background'
+        annotations = np.squeeze(prediction_array)
         self.image_size_z, self.image_size_rows, self.image_size_cols = annotations.shape[:3]
         self.ROI_Names = ROI_Names
         self.output_dir = output_dir
@@ -494,6 +501,9 @@ class DicomReaderWriter:
         self.annotations = annotations
         self.Mask_to_Contours()
 
+    def with_annotations(self, annotations, output_dir, ROI_Names=None):
+        self.prediction_array_to_RT(prediction_array=annotations, output_dir=output_dir, ROI_Names=ROI_Names)
+
     def Mask_to_Contours(self):
         self.RefDs = self.ds
         self.PixelSize = self.dicom_handle.GetSpacing()
@@ -557,9 +567,7 @@ class DicomReaderWriter:
             q = Queue(maxsize=thread_count)
             threads = []
             kwargs = {'image_size_rows': self.image_size_rows, 'image_size_cols': self.image_size_cols,
-                      'slice_info': self.slice_info, 'PixelSize': self.PixelSize, 'mv': self.mv,
-                      'shift_list': self.shift_list, 'contour_dict': contour_dict,
-                      'RS': self.RS_struct, 'ds': self.ds}
+                      'PixelSize': self.PixelSize, 'contour_dict': contour_dict, 'RS': self.RS_struct}
 
             A = [q,kwargs]
             # pointer_class = Point_Output_Maker_Class(**kwargs)
@@ -703,8 +711,8 @@ class DicomReaderWriter:
             self.get_dose()
         true_rois = []
         for roi in self.rois_in_case:
-            if roi not in self.all_rois:
-                self.all_rois.append(roi)
+            if roi.lower() not in self.all_rois:
+                self.all_rois.append(roi.lower())
             if self.Contour_Names:
                 if roi.lower() in self.associations:
                     true_rois.append(self.associations[roi.lower()])
